@@ -3,8 +3,8 @@
  *
  */
 char acct_sccsid[] =
-"@(#)acct.c	2.13 Copyright 1994 Livingston Enterprises Inc\n"
-"		     Copyright 1999 Cistron Internet Services B.V.";
+"@(#)acct.c	2.16 Copyright 1994 Livingston Enterprises Inc\n"
+"		     Copyright 1999,2000 Cistron Internet Services B.V.";
 
 
 #include	<sys/types.h>
@@ -28,19 +28,12 @@ char acct_sccsid[] =
 #include	"radiusd.h"
 #include	"radutmp.h"
 
-/*
- *	FIXME: this should be configurable.
- */
-int	doradwtmp = 1;
-
 static char trans[64] =
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 #define ENC(c) trans[c]
 
 static char porttypes[] = "ASITX";
 
-#define LOCK_LEN sizeof(struct radutmp)
-
 /*
  *	used for caching radutmp lookups.
  */
@@ -122,13 +115,14 @@ static void make_wtmp(struct radutmp *ut, struct utmp *wt, int status)
 	 *	and address so that the tty field is unique.
 	 */
 	s = "";
-	if ((cl = nas_find(ntohl(ut->nas_address))) != NULL)
+	if ((cl = nas_find(ntohl(ut->nas_address))) != NULL &&
+	    strcmp(cl->longname, "DEFAULT") != 0)
 		s = cl->shortname;
 	if (s == NULL || s[0] == 0) s = uue(&(ut->nas_address));
 #if UT_LINESIZE > 9
-	sprintf(buf, "%03d:%s", ut->nas_port, s);
+	sprintf(buf, "%03d:%.20s", ut->nas_port, s);
 #else
-	sprintf(buf, "%02d%s", ut->nas_port, s);
+	sprintf(buf, "%02d%.20s", ut->nas_port, s);
 #endif
 	strncpy(wt->ut_line, buf, UT_LINESIZE);
 
@@ -167,7 +161,7 @@ static void make_wtmp(struct radutmp *ut, struct utmp *wt, int status)
 /*
  *	Zap a user, or all users on a NAS, from the radutmp file.
  */
-int radzap(UINT4 nasaddr, int port, char *user, time_t t)
+int radzap(UINT4 nasaddr, int port, char *user, time_t t, int dowtmp)
 {
 	struct radutmp	u;
 	struct utmp	wt;
@@ -176,7 +170,10 @@ int radzap(UINT4 nasaddr, int port, char *user, time_t t)
 	UINT4		netaddr;
 
 	if (t == 0) time(&t);
-	fp = fopen(RADWTMP, "a");
+	if (dowtmp)
+		fp = fopen(RADWTMP, "a");
+	else
+		fp = NULL;
 	netaddr = htonl(nasaddr);
 
 	if ((fd = open(RADUTMP, O_RDWR|O_CREAT, 0644)) >= 0) {
@@ -186,7 +183,7 @@ int radzap(UINT4 nasaddr, int port, char *user, time_t t)
 		 *	Lock the utmp file, prefer lockf() over flock().
 		 */
 #if defined(F_LOCK) && !defined(BSD)
-		(void)lockf(fd, F_LOCK, LOCK_LEN);
+		(void)lockf(fd, F_LOCK, 0);
 #else
 		(void)flock(fd, LOCK_EX);
 #endif
@@ -356,6 +353,14 @@ int rad_accounting_new(AUTH_REQ *authreq, int dowtmp)
 				 *	last 8 bytes.
 				 */
 				off = vp->length - sizeof(ut.session_id);
+				/*
+				 *	Ascend is br0ken - it adds a \0
+				 *	to the end of any string.
+				 *	Compensate.
+				 */
+				if (vp->length > 0 &&
+				    vp->strvalue[vp->length - 1] == 0)
+					off--;
 				if (off < 0) off = 0;
 				memcpy(ut.session_id, vp->strvalue + off,
 					sizeof(ut.session_id));
@@ -368,7 +373,7 @@ int rad_accounting_new(AUTH_REQ *authreq, int dowtmp)
 			case PW_CALLING_STATION_ID:
 				strncpy(ut.caller_id, vp->strvalue,
 					sizeof(ut.caller_id));
-				ut.caller_id[sizeof(ut.caller_id)] = 0;
+				ut.caller_id[sizeof(ut.caller_id) - 1] = 0;
 				break;
 		}
 	}
@@ -397,13 +402,13 @@ int rad_accounting_new(AUTH_REQ *authreq, int dowtmp)
 	if (status == PW_STATUS_ACCOUNTING_ON && nas_address) {
 		log(L_INFO, "NAS %s restarted (Accounting-On packet seen)",
 			nas_name(nas_address));
-		radzap(nas_address, -1, NULL, ut.time);
+		radzap(nas_address, -1, NULL, ut.time, dowtmp);
 		return 0;
 	}
 	if (status == PW_STATUS_ACCOUNTING_OFF && nas_address) {
 		log(L_INFO, "NAS %s rebooted (Accounting-Off packet seen)",
 			nas_name(nas_address));
-		radzap(nas_address, -1, NULL, ut.time);
+		radzap(nas_address, -1, NULL, ut.time, dowtmp);
 		return 0;
 	}
 
@@ -445,7 +450,7 @@ int rad_accounting_new(AUTH_REQ *authreq, int dowtmp)
 		 *	Lock the utmp file, prefer lockf() over flock().
 		 */
 #if defined(F_LOCK) && !defined(BSD)
-		(void)lockf(fd, F_LOCK, LOCK_LEN);
+		(void)lockf(fd, F_LOCK, 0);
 #else
 		(void)flock(fd, LOCK_EX);
 #endif
@@ -590,6 +595,7 @@ int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f)
 	VALUE_PAIR	*pair;
 	UINT4		nas;
 	NAS		*cl;
+	int		flat = 0;
 	long		curtime;
 	int		ret = 0;
 	struct stat	st;
@@ -601,50 +607,68 @@ int rad_accounting_orig(AUTH_REQ *authreq, int authtype, char *f)
 	if (stat(radacct_dir, &st) < 0)
 		return 0;
 	curtime = time(0);
+	nasname[0] = 0;
 
 	/*
-	 *	Find out the name of this terminal server. We try
-	 *	to find the PW_NAS_IP_ADDRESS in the naslist file.
-	 *	If that fails, we look for the originating address.
-	 *	Only if that fails we resort to a name lookup.
+	 *	If radacct_dir ends in '/', assume we want a flat
+	 *	accouting directory (no subdirs, one detail file).
 	 */
-	cl = NULL;
-	nas = authreq->ipaddr;
-	if ((pair = pairfind(authreq->request, PW_NAS_IP_ADDRESS)) != NULL)
-		nas = pair->lvalue;
-	if (authreq->server_ipaddr)
-		nas = authreq->server_ipaddr;
+	if ((s = strrchr(radacct_dir, '/')) != NULL && s[1] == 0)
+		flat = 1;
 
-	if ((cl = nas_find(nas)) != NULL) {
-		/* shortname and longname are < 128 bytes */
-		if (cl->shortname[0])
-			strcpy(nasname, cl->shortname);
-		else
-			strcpy(nasname, cl->longname);
+	if (!flat) {
+		/*
+		 *	Find out the name of this terminal server. We try
+		 *	to find the PW_NAS_IP_ADDRESS in the naslist file.
+		 *	If that fails, we look for the originating address.
+		 *	Only if that fails we resort to a name lookup.
+		 */
+		cl = NULL;
+		nas = authreq->ipaddr;
+		if ((pair = pairfind(authreq->request, PW_NAS_IP_ADDRESS)) != NULL)
+			nas = pair->lvalue;
+
+		/* Reply from remote radius server: use server_ipaddr */
+		if (authreq->server_ipaddr)
+			nas = authreq->server_ipaddr;
+
+		if ((cl = nas_find(nas)) != NULL) {
+			/* shortname and longname are < 128 bytes */
+			if (cl->shortname[0])
+				strcpy(nasname, cl->shortname);
+			else
+				strcpy(nasname, cl->longname);
+		}
+
+		if (cl == NULL) {
+			s = ip_hostname(nas);
+			if (strlen(s) >= sizeof(nasname) || strchr(s, '/'))
+				return -1;
+			strcpy(nasname, s);
+		}
+
+		/*
+		 *	Create a directory for this nas.
+		 */
+		sprintf(buffer, "%.250s/%.250s", radacct_dir, nasname);
+		(void) mkdir(buffer, 0755);
 	}
 
-	if (cl == NULL) {
-		s = ip_hostname(nas);
-		if (strlen(s) >= sizeof(nasname) || strchr(s, '/'))
-			return -1;
-		strcpy(nasname, s);
-	}
-
-	/*
-	 *	Create a directory for this nas.
-	 */
-	sprintf(buffer, "%.250s/%.250s", radacct_dir, nasname);
-	(void) mkdir(buffer, 0755);
-
 	/*
 	 *	Write Detail file.
 	 */
-	sprintf(buffer, "%.200s/%.200s/%.100s", radacct_dir, nasname, f ? f : "detail");
-	if((outfd = fopen(buffer, "a")) == (FILE *)NULL) {
+	sprintf(buffer, "%.200s/%.200s%s%.100s", radacct_dir,
+		nasname, nasname[0] ? "/" : "", f ? f : "detail");
+	if ((outfd = fopen(buffer, "a")) == NULL) {
 		log(L_ERR, "Acct: Couldn't open file %s", buffer);
 		ret = -1;
 	} else {
 
+#if defined(F_LOCK) && !defined(BSD)
+		(void)lockf(fileno(outfd), F_LOCK, 0);
+#else
+		(void)flock(fileno(outfd), LOCK_EX);
+#endif
 		/* Post a timestamp */
 		fputs(ctime(&curtime), outfd);
 
@@ -710,7 +734,7 @@ int rad_accounting(AUTH_REQ *authreq, int activefd)
 		 *	we run it first, the stripped info will also
 		 *	get into the "details" file.
 		 */
-		if (rad_accounting_new(authreq, doradwtmp) == 0)
+		if (rad_accounting_new(authreq, use_wtmp) == 0)
 			reply = 1;
 		if (rad_accounting_orig(authreq, auth, NULL) == 0)
 			reply = 1;
@@ -721,7 +745,7 @@ int rad_accounting(AUTH_REQ *authreq, int activefd)
 		 */
 		if (rad_accounting_orig(authreq, auth, NULL) == 0)
 			reply = 1;
-		if (rad_accounting_new(authreq, doradwtmp) == 0)
+		if (rad_accounting_new(authreq, use_wtmp) == 0)
 			reply = 1;
 	}
 
@@ -881,7 +905,7 @@ int rad_check_multi(char *name, VALUE_PAIR *request, int maxsimul)
 	 *	lockf() the file while reading/writing.
 	 */
 #if defined(F_LOCK) && !defined(BSD)
-		(void)lockf(fd, F_LOCK, LOCK_LEN);
+		(void)lockf(fd, F_LOCK, 0);
 #else
 		(void)flock(fd, LOCK_EX);
 #endif
