@@ -2,11 +2,11 @@
  * auth.c	User authentication.
  *
  *
- * Version:	@(#)auth.c  1.86  06-Oct-1999  miquels@cistron.nl
+ * Version:	@(#)auth.c  1.89  30-Aug-2000  miquels@cistron.nl
  *
  */
 char auth_sccsid[] =
-"@(#)auth.c	1.86 Copyright 1998-1999 Cistron Internet Services B.V.";
+"@(#)auth.c	1.89 Copyright 1998-2000 Cistron Internet Services B.V.";
 
 #include	<sys/types.h>
 #include	<sys/time.h>
@@ -116,12 +116,8 @@ static int unix_pass(char *name, char *passwd)
 	char		*encrypted_pass;
 	int		ret;
 #if !defined(NOSHADOW)
-#if defined(M_UNIX)
-	struct passwd	*spwd;
-#else
 	struct spwd	*spwd;
 #endif
-#endif /* !NOSHADOW */
 #ifdef OSFC2
 	struct pr_passwd *pr_pw;
 #endif
@@ -143,18 +139,6 @@ static int unix_pass(char *name, char *passwd)
 	encrypted_pass = pwd->pw_passwd;
 #endif /* OSFC2 */
 
-#if !defined(NOSHADOW)
-	/*
-	 *      See if there is a shadow password.
-	 */
-	if ((spwd = getspnam(name)) != NULL)
-#if defined(M_UNIX)
-		encrypted_pass = spwd->pw_passwd;
-#else
-		encrypted_pass = spwd->sp_pwdp;
-#endif	/* M_UNIX */
-#endif	/* !NOSHADOW */
-#
 #ifdef DENY_SHELL
 	/*
 	 *	Undocumented temporary compatibility for iphil.NET
@@ -166,15 +150,30 @@ static int unix_pass(char *name, char *passwd)
 	}
 #endif
 
-#if !defined(NOSHADOW) && !defined(M_UNIX)
+#if !defined(NOSHADOW)
+	/*
+	 *      See if there is a shadow password.
+	 */
+	if ((spwd = getspnam(name)) != NULL)
+		encrypted_pass = spwd->sp_pwdp;
+
 	/*
 	 *      Check if password has expired.
 	 */
-	if (spwd && spwd->sp_expire > 0 &&
-	    (time(NULL) / 86400) > spwd->sp_expire) {
+	if (spwd && spwd->sp_lstchg >= 0 && spwd->sp_max >= 0 &&
+	    (time(NULL) / 86400) > (spwd->sp_lstchg + spwd->sp_max)) {
 		log(L_AUTH, "unix_pass: [%s]: password has expired", name);
 		return -1;
 	}
+
+	/*
+	 *      Check if account has expired.
+	 */
+	if (spwd && spwd->sp_expire >= 0 &&
+	    (time(NULL) / 86400) > spwd->sp_expire) {
+		log(L_AUTH, "unix_pass: [%s]: account has expired", name);
+		return -1;
+	}
 #endif
 
 #if defined(__FreeBSD__) || defined(bsdi) || defined(_PWF_EXPIRE)
@@ -215,7 +214,7 @@ static int unix_pass(char *name, char *passwd)
 
 	  while ((shell = getusershell()) != NULL) {
 	    if (strcmp(shell, pwd->pw_shell) == 0 ||
-		strcpy(shell, "/RADIUSD/ANY/SHELL") == 0) {
+		strcmp(shell, "/RADIUSD/ANY/SHELL") == 0) {
 	      endusershell();
 	      return 0;
 	    }
@@ -451,15 +450,16 @@ static int rad_check_password(AUTH_REQ *authreq, int activefd,
 				memcpy(ptr, authreq->vector, AUTH_VECTOR_LEN);
 				i += AUTH_VECTOR_LEN;
 			}
-			md5_calc(pw_digest, string, i);
+			md5_calc(pw_digest, (unsigned char *)string, i);
 
 			/*
 			 *	Compare them
 			 */
 			if (memcmp(pw_digest, auth_item->strvalue + 1,
-					CHAP_VALUE_LENGTH) != 0)
+				   CHAP_VALUE_LENGTH) != 0) {
+				strcpy(userpass, "<BAD-CHAP-PASSWORD>");
 				result = -1;
-			else
+			} else
 				strcpy(userpass, password_pair->strvalue);
 			break;
 		default:
@@ -492,9 +492,7 @@ int rad_auth_init(AUTH_REQ *authreq, int activefd)
 	   (strlen(namepair->strvalue) <= 0)) {
 		log(L_ERR, "No username: [] (from nas %s)",
 			nas_name2(authreq));
-		pairfree(authreq->request);
-		memset(authreq, 0, sizeof(AUTH_REQ));
-		free(authreq);
+		authfree(authreq);
 		return -1;
 	}
 
@@ -597,8 +595,10 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 	if (authreq->server_code == PW_AUTHENTICATION_ACK)
 		user_check->lvalue = PW_AUTHTYPE_ACCEPT;
 
-	if (authreq->server_reply)
+	if (authreq->server_reply) {
 		user_reply = authreq->server_reply;
+		authreq->server_reply = NULL;
+	}
 
 
 	/*
@@ -606,8 +606,10 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 	 */
 	if (user_find(namepair->strvalue, authreq->request,
 	   &user_check, &user_reply) != 0) {
-		log(L_AUTH, "Invalid user: [%s] (%s)",
+		log(L_AUTH, "Unknown user: [%s] (%s)",
 			namepair->strvalue, auth_name(authreq, 1));
+		DEBUG2("The user is NOT configured to have any reply attributes.  "
+		       "If you are proxying, please read doc/README.proxy.");
 		rad_send_reply(PW_AUTHENTICATION_REJECT, authreq,
 			NULL, NULL, activefd);
 		pairfree(user_reply);
@@ -645,9 +647,11 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 			user_reply, user_msg, activefd);
 		if (log_auth) {
 			log(L_AUTH,
-				"Login incorrect: [%s/%s] (%s)",
-				namepair->strvalue, userpass,
-				auth_name(authreq, 1));
+				"Login incorrect: [%s/%s] (%s)%s",
+				namepair->strvalue,
+				userpass[0] ? userpass : "<NO-PASSWORD>",
+				auth_name(authreq, 1),
+				((result == -2) ? " reject" : ""));
 		}
 	}
 
