@@ -9,7 +9,7 @@
  *	Pleasanton, CA   94566
  *
  *	Copyright 1992 Livingston Enterprises, Inc.
- *	Copyright 1996-1999 Cistron Internet Services B.V.
+ *	Copyright 1996-2000 Cistron Internet Services B.V.
  *
  *	Permission to use, copy, modify, and distribute this software for any
  *	purpose and without fee is hereby granted, provided that this
@@ -28,7 +28,7 @@
 
 char files_sccsid[] =
 "@(#)users.c	1.12 Copyright 1992 Livingston Enterprises Inc\n"
-"@(#)files.c	2.42 Copyright 1999 Cistron Internet Services B.V.";
+"@(#)files.c	2.44 Copyright 2000 Cistron Internet Services B.V.";
 
 #include	<sys/types.h>
 #include	<sys/socket.h>
@@ -44,6 +44,7 @@ char files_sccsid[] =
 #include	<time.h>
 #include	<ctype.h>
 #include	<fcntl.h>
+#include	<errno.h>
 
 #ifdef DBM
 #  include	<dbm.h>
@@ -81,6 +82,8 @@ CLIENT		*clients;
 NAS		*naslist;
 REALM		*realms;
 
+#define USERPARSE_EOS	(0)
+#define USERPARSE_COMMA (1)
 
 /*
  *	Returns a copy of a pair list.
@@ -162,18 +165,36 @@ void pairmove(VALUE_PAIR **to, VALUE_PAIR **from)
 			tailfrom = i;
 			continue;
 		}
+
 		/*
-		 *	If the attribute is already present in "to",
-		 *	do not move it from "from" to "to". We make
-		 *	an exception for "Hint" which can appear multiple
-		 *	times, and we never move "Fall-Through".
+		 *	We never move "Fall-Through" to the "to" list.
 		 */
-		if (i->attribute == PW_FALL_THROUGH ||
-		    (i->attribute != PW_HINT && i->attribute != PW_FRAMED_ROUTE
-		     && pairfind(*to, i->attribute) != 0)) {
+		if (i->attribute == PW_FALL_THROUGH) {
 			tailfrom = i;
 			continue;
 		}
+
+#if 1
+		/*
+		 *	We always move "Hint" and "Framed-Route" to the
+		 *	"to" list.
+		 *
+		 *	If the attribute is already present in "to",
+		 *	do not move it from "from" to "to".
+		 *
+		 *	NOTE: THIS MAY CAUSE PROBLEMS FOR YOU.
+		 *	Comment out this section of code if you don't
+		 *	like it, by changing the previous
+		 *	'#if 1' to '#if 0'
+		 */
+		if ((i->attribute != PW_HINT) &&
+		    (i->attribute != PW_FRAMED_ROUTE) &&
+		    (pairfind(*to, i->attribute) != NULL)) {
+			DEBUG2("WARNING: Duplicate attribute %s is being ignored!", i->name);
+			tailfrom = i;
+			continue;
+		}
+#endif
 		if (tailfrom)
 			tailfrom->next = next;
 		else
@@ -498,11 +519,6 @@ static int paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 						return -1;
 				}
 				else
-				if (check_item->attribute == PW_NAS_PORT_ID) {
-					if (portcmp(check_item, auth_item) != 0)
-						return -1;
-				}
-			 	else
 				if (check_item->attribute == PW_GROUP_NAME ||
 				    check_item->attribute == PW_GROUP) {
 					compare = groupcmp(check_item, username);
@@ -520,6 +536,11 @@ static int paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 				break;
 
 			case PW_TYPE_INTEGER:
+				if (check_item->attribute == PW_NAS_PORT_ID) {
+					compare = portcmp(check_item,auth_item);
+					break;
+				}
+				/*FALLTHRU*/
 			case PW_TYPE_IPADDR:
 				compare = auth_item->lvalue - check_item->lvalue;
 				break;
@@ -621,11 +642,6 @@ static int hunt_paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 		switch(check_item->type) {
 
 			case PW_TYPE_STRING:
-				if (check_item->attribute == PW_NAS_PORT_ID) {
-					if (portcmp(check_item, auth_item) == 0)
-						result = 0;
-				}
-			 	else
 				if (check_item->attribute == PW_GROUP_NAME ||
 				    check_item->attribute == PW_GROUP) {
 					if (groupcmp(check_item, auth_item->strvalue)==0)
@@ -645,6 +661,12 @@ static int hunt_paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 				break;
 
 			case PW_TYPE_INTEGER:
+				if (check_item->attribute == PW_NAS_PORT_ID) {
+					if (portcmp(check_item, auth_item) == 0)
+						result = 0;
+					break;
+				}
+				/*FALLTHRU*/
 			case PW_TYPE_IPADDR:
 				if(check_item->lvalue == auth_item->lvalue) {
 					result = 0;
@@ -807,7 +829,7 @@ static void pair_sanity(char *fn, PAIR_LIST *entry)
  *	Read the users, huntgroups or hints file.
  *	Return a PAIR_LIST.
  */
-static PAIR_LIST *file_read(char *file, int complain)
+static PAIR_LIST *file_read(char *file, int *err)
 {
 	FILE		*fp;
 	int		mode = FIND_MODE_NAME;
@@ -819,28 +841,44 @@ static PAIR_LIST *file_read(char *file, int complain)
 	PAIR_LIST	*pl = NULL, *last = NULL, *t;
 	int		lineno = 0;
 	int		old_lineno;
+	int		parsecode;
+
+	*err = 1;
 
 	/*
 	 *	Open the table
 	 */
 	if ((fp = fopen(file, "r")) == NULL) {
-		if (!complain) return NULL;
-		log(L_CONS|L_ERR, "Couldn't open %s for reading", file);
+		log(L_CONS|L_ERR, "Couldn't open %s for reading: %s",
+		    file, strerror(errno));
 		return NULL;
 	}
 
+	parsecode = USERPARSE_EOS;
 	/*
 	 *	Read the entire file into memory for speed.
 	 */
 	while(fgets(buffer, sizeof(buffer), fp) != (char *)NULL) {
 		lineno++;
+
 		if (buffer[0] == '#' || buffer[0] == '\n') continue;
+
 parse_again:
 		if(mode == FIND_MODE_NAME) {
 			/*
-			 *	Find the entry starting with the users name
+			 *	Find the entry starting with the users name,
+			 *	ignoring whitespace.
 			 */
-			if (isspace(buffer[0])) continue;
+			if (isspace(buffer[0])) {
+				if (parsecode != USERPARSE_EOS) {
+					log(L_ERR|L_CONS,
+					    "%s[%d]: Unexpected trailing comma for entry %s",
+					    file, lineno, entry);
+					fclose(fp);
+					return NULL;
+				}
+				continue;
+			}
 
 			/*
 			 *	FIXME:	support `\ ' space in usernames.
@@ -866,8 +904,10 @@ parse_again:
 				while (!isspace(*ptr))
 					ptr++;
 				*ptr = 0;
-				if ((t = file_read(s, 1)) == NULL)
-					continue;
+				if ((t = file_read(s, err)) == NULL && err) {
+					fclose(fp);
+					return NULL;
+				}
 				if (last)
 					last->next = t;
 				else
@@ -884,21 +924,38 @@ parse_again:
 			check_tmp = NULL;
 			reply_tmp = NULL;
 			old_lineno = lineno;
-			if(userparse(ptr, &check_tmp) != 0) {
+			parsecode = userparse(ptr, &check_tmp);
+			if(parsecode < 0) {
 				log(L_ERR|L_CONS,
 				"%s[%d]: Parse error (check) for entry %s",
 					file, lineno, entry);
 				fclose(fp);
 				return NULL;
+			} else if (parsecode == USERPARSE_COMMA) {
+				log(L_ERR|L_CONS,
+				    "%s[%d]: Unexpected trailing comma in check item list for entry %s",
+				    file, lineno, entry);
+				fclose(fp);
+				return NULL;
 			}
 			mode = FIND_MODE_REPLY;
-		}
+			parsecode = USERPARSE_COMMA;
+		} /* mode == FIND_MODE_NAME */
 		else {
 			if(*buffer == ' ' || *buffer == '\t') {
+				if (parsecode != USERPARSE_COMMA) {
+					log(L_ERR|L_CONS,
+				"%s[%d]: Syntax error: Previous line is missing a trailing comma for entry %s",
+						file, lineno, entry);
+					fclose(fp);
+					return NULL;
+				}
+
 				/*
 				 *	Parse the reply values
 				 */
-				if (userparse(buffer, &reply_tmp)!=0) {
+				parsecode = userparse(buffer, &reply_tmp);
+				if (parsecode < 0) {
 					log(L_ERR|L_CONS,
 				"%s[%d]: Parse error (reply) for entry %s",
 						file, lineno, entry);
@@ -945,6 +1002,7 @@ parse_again:
 	}
 	fclose(fp);
 
+	*err = 0;
 	return pl;
 }
 
@@ -1269,15 +1327,22 @@ int userparse(char *buffer, VALUE_PAIR **first_pair)
 	struct tm	*tm;
 	time_t		timeval;
 	int		operator;
+	int		rcode;
 
+	rcode = USERPARSE_EOS;
 	mode = PARSE_MODE_NAME;
 	while(*buffer != '\n' && *buffer != '\0') {
 
+		if (*buffer == ',') {
+			rcode = USERPARSE_COMMA;
+		}
+
 		if(*buffer == ' ' || *buffer == '\t' || *buffer == ',') {
 			buffer++;
 			continue;
 		}
 
+		rcode = USERPARSE_EOS;
 		switch(mode) {
 
 		case PARSE_MODE_NAME:
@@ -1285,10 +1350,8 @@ int userparse(char *buffer, VALUE_PAIR **first_pair)
 			fieldcpy(attrstr, sizeof(attrstr), &buffer);
 			if((attr = dict_attrfind(attrstr)) ==
 						(DICT_ATTR *)NULL) {
-#if 0 /* Be quiet. */
-				log(L_ERR|L_CONS, "unknown attribute %s",
+				log(L_ERR|L_CONS, "Unknown attribute \"%s\"",
 					attrstr);
-#endif
 				return(-1);
 			}
 			mode = PARSE_MODE_EQUAL;
@@ -1327,7 +1390,8 @@ int userparse(char *buffer, VALUE_PAIR **first_pair)
 				operator = PW_OPERATOR_GREATER_EQUAL;
 			}
 			else {
-				log(L_ERR|L_CONS, "expected `='");
+				log(L_ERR|L_CONS, "Unexpected character `%c' (0x%02x)",
+					*buffer, *buffer);
 				return(-1);
 			}
 			break;
@@ -1346,26 +1410,21 @@ int userparse(char *buffer, VALUE_PAIR **first_pair)
 			pair->type = attr->type;
 			pair->operator = operator;
 
+			/* _Always_ copy value as string. */
+			strNcpy(pair->strvalue, valstr, sizeof(pair->strvalue));
+
 			switch(pair->type) {
 
 			case PW_TYPE_STRING:
-do_copy:
-				strncpy(pair->strvalue, valstr,
-					sizeof(pair->strvalue));
-				pair->strvalue[sizeof(pair->strvalue) - 1] = 0;
+				/* Pair->strvalue has been set already */
 				pair->length = strlen(pair->strvalue);
 				break;
 
 			case PW_TYPE_INTEGER:
 				/*
-				 *	For PW_NAS_PORT_ID, allow a
-				 *	port range instead of just a port.
+				 *	If it starts with a digit, it must
+				 *	be a number (or a range).
 				 */
-				if (attr->value == PW_NAS_PORT_ID) {
-					for(s = valstr; *s; s++)
-						if (!isdigit(*s)) break;
-					if (*s) goto do_copy;
-				}
 				if (isdigit(*valstr)) {
 					pair->lvalue = atoi(valstr);
 					pair->length = 4;
@@ -1373,7 +1432,7 @@ do_copy:
 				else if((dval = dict_valfind(valstr)) ==
 							(DICT_VALUE *)NULL) {
 					free(pair);
-					log(L_ERR|L_CONS, "unknown value %s",
+					log(L_ERR|L_CONS, "Unknown value \"%s\"",
 						valstr);
 					return(-1);
 				}
@@ -1436,10 +1495,8 @@ do_copy:
 
 			default:
 				free(pair);
-#if 0 /* Yeah yeah */
-				log(L_ERR|L_CONS, "unknown attr. type %d",
+				log(L_ERR|L_CONS, "Unknown attribute type %d",
 					pair->type);
-#endif
 				return(-1);
 			}
 			pairadd(first_pair, pair);
@@ -1451,7 +1508,16 @@ do_copy:
 			break;
 		}
 	}
-	return(0);
+
+	/*
+	 *	Double-check that we've parsed a complete Attribute=value
+	 */
+	if (mode != PARSE_MODE_NAME) {
+		log(L_ERR|L_CONS, "Got end of line while still parsing last attribute");
+		return(-1);
+	}
+
+	return rcode;
 }
 
 /*
@@ -1764,7 +1830,7 @@ int read_clients_file(char *file)
 	clients = NULL;
 
 	if ((fp = fopen(file, "r")) == NULL) {
-		log(L_CONS|L_ERR, "cannot open %s", file);
+		log(L_CONS|L_ERR, "cannot open %s: %s", file, strerror(errno));
 		return -1;
 	}
 	while(fgets(buffer, 256, fp) != NULL) {
@@ -1870,13 +1936,14 @@ int read_naslist_file(char *file)
 	char	shortnm[32];
 	char	nastype[32];
 	int	lineno = 0;
+	int	dotted;
 	NAS	*c;
 
 	nas_free(naslist);
 	naslist = NULL;
 
 	if ((fp = fopen(file, "r")) == NULL) {
-		log(L_CONS|L_ERR, "cannot open %s", file);
+		log(L_CONS|L_ERR, "cannot open %s: %s", file, strerror(errno));
 		return -1;
 	}
 	while(fgets(buffer, 256, fp) != NULL) {
@@ -1886,18 +1953,23 @@ int read_naslist_file(char *file)
 		nastype[0] = 0;
 		if (sscanf(buffer, "%127s%31s%31s", hostnm, shortnm, nastype) < 2) {
 			log(L_ERR, "%s[%d]: syntax error", file, lineno);
-			continue;
+			return -1;
 		}
 		if ((c = malloc(sizeof(NAS))) == NULL) {
 			log(L_CONS|L_ERR, "%s[%d]: out of memory",
 				file, lineno);
 			return -1;
 		}
+		c->ipaddr = 0;
 
-		c->ipaddr = get_ipaddr(hostnm);
+		dotted = 0;
+		if (strcmp(hostnm, "DEFAULT") != 0) {
+			c->ipaddr = get_ipaddr(hostnm);
+			dotted = good_ipaddr(hostnm);
+		}
 		strNcpy(c->nastype, nastype, sizeof(c->nastype));
 		strNcpy(c->shortname, shortnm, sizeof(c->shortname));
-		strNcpy(c->longname, ip_hostname(c->ipaddr),
+		strNcpy(c->longname, dotted ? ip_hostname(c->ipaddr) : hostnm,
 			sizeof(c->longname));
 
 		c->next = naslist;
@@ -1917,8 +1989,9 @@ int read_realms_file(char *file)
 	char	buffer[256];
 	char	realm[32];
 	char	hostnm[128];
-	char	opts1[32], opts2[32];
+	char	opts[5][32];
 	char	*p;
+	int	i;
 	int	lineno = 0;
 	REALM	*c;
 
@@ -1926,28 +1999,30 @@ int read_realms_file(char *file)
 	realms = NULL;
 
 	if ((fp = fopen(file, "r")) == NULL) {
-#if 1 /* For now - realms file is not obligatory */
-		return 0;
-#else
-		log(L_CONS|L_ERR, "cannot open %s", file);
+		log(L_CONS|L_ERR, "cannot open %s: %s", file, strerror(errno));
 		return -1;
-#endif
 	}
 	while(fgets(buffer, 256, fp) != NULL) {
 		lineno++;
 		if (buffer[0] == '#' || buffer[0] == '\n')
 			continue;
-		opts1[0] = opts2[0] = 0;
-		if (sscanf(buffer, "%31s%127s%31s%31s",
-		    realm, hostnm, opts1, opts2) < 2) {
+		opts[0][0] = opts[1][0] = 0;
+		opts[2][0] = opts[3][0] = 0;
+		opts[4][0] = 0;
+		if (sscanf(buffer, "%31s%127s%31s%31s%31s%31s%31s",
+		    realm, hostnm, opts[0], opts[1],
+		    	opts[2], opts[3], opts[4]) < 2) {
 			log(L_ERR, "%s[%d]: syntax error", file, lineno);
-			continue;
+			return -1;
 		}
 		if ((c = malloc(sizeof(REALM))) == NULL) {
 			log(L_CONS|L_ERR, "%s[%d]: out of memory",
 				file, lineno);
 			return -1;
 		}
+		memset(c, 0, sizeof(REALM));
+		c->striprealm = 1;
+		c->dohints = 0;
 
 		if ((p = strchr(hostnm, ':')) != NULL) {
 			*p++ = 0;
@@ -1961,11 +2036,20 @@ int read_realms_file(char *file)
 			c->ipaddr = get_ipaddr(hostnm);
 		strNcpy(c->realm, realm, sizeof(c->realm));
 		strNcpy(c->server, hostnm, sizeof(c->server));
+
 		/* Yes we could do this more intelligently */
-		c->striprealm = strcmp(opts1, "nostrip") != 0 ||
-				strcmp(opts2, "nostrip") != 0;
-		c->dohints    = strcmp(opts1, "hints") == 0 ||
-				strcmp(opts2, "hints") == 0;
+		for(i = 0; i < 3; i++) {
+			if (strcmp(opts[i], "nostrip") == 0)
+				c->striprealm = 0;
+			if (strcmp(opts[i], "hints") == 0)
+				c->dohints = 1;
+			if (strcmp(opts[i], "noacct") == 0)
+				c->acct_port = 0;
+			if (strcmp(opts[i], "noauth") == 0)
+				c->auth_port = 0;
+			if (strcmp(opts[i], "trusted") == 0)
+				c->trusted = 0;
+		}
 		c->next = realms;
 		realms = c;
 	}
@@ -1998,13 +2082,17 @@ REALM *realm_find(char *realm)
  */
 NAS *nas_find(UINT4 ipaddr)
 {
-	NAS *cl;
+	NAS *cl, *dfl;
 
-	for(cl = naslist; cl; cl = cl->next)
+	dfl = NULL;
+	for(cl = naslist; cl; cl = cl->next) {
 		if (ipaddr == cl->ipaddr)
 			break;
+		if (strcmp(cl->longname, "DEFAULT") == 0)
+			dfl = cl;
+	}
 
-	return cl;
+	return cl ? cl : dfl;
 }
 
 
@@ -2015,7 +2103,8 @@ char *nas_name(UINT4 ipaddr)
 {
 	NAS *cl;
 
-	if ((cl = nas_find(ipaddr)) != NULL) {
+	if ((cl = nas_find(ipaddr)) != NULL &&
+	    strcmp(cl->longname, "DEFAULT") != 0) {
 		if (cl->shortname[0])
 			return cl->shortname;
 		else
@@ -2039,7 +2128,8 @@ char *nas_name2(AUTH_REQ *authreq)
 	else
 		ipaddr = authreq->ipaddr;
 
-	if ((cl = nas_find(ipaddr)) != NULL) {
+	if ((cl = nas_find(ipaddr)) != NULL &&
+	    strcmp(cl->longname, "DEFAULT") != 0) {
 		if (cl->shortname[0])
 			return cl->shortname;
 		else
@@ -2091,7 +2181,9 @@ static int checkdbm(char *users, char *ext)
  */
 int read_config_files()
 {
-	char buffer[256];
+	struct stat	st;
+	char		buffer[256];
+	int		err;
 
 	pairlist_free(&users);
 	pairlist_free(&huntgroups);
@@ -2104,22 +2196,33 @@ int read_config_files()
 		 checkdbm(users->name, ".db") == 0))
 			log(L_INFO|L_CONS, "DBM files found but no -b flag given - NOT using DBM");
 #endif
-	if (!use_dbm && (users = file_read(buffer, 1)) == NULL)
+	if (!use_dbm && (users = file_read(buffer, &err)) == NULL)
 		return -1;
 
 	sprintf(buffer, "%.200s/%.50s", radius_dir, RADIUS_HUNTGROUPS);
-	huntgroups = file_read(buffer, 0);
+	if (stat(buffer, &st) == 0 || errno != ENOENT) {
+		if ((huntgroups = file_read(buffer, &err)) == NULL && err)
+			return -1;
+	}
 	sprintf(buffer, "%.200s/%.50s", radius_dir, RADIUS_HINTS);
-	hints      = file_read(buffer, 0);
+	if (stat(buffer, &st) == 0 || errno != ENOENT) {
+		if ((hints = file_read(buffer, &err)) == NULL && err)
+			return -1;
+	}
+
 	sprintf(buffer, "%.200s/%.50s", radius_dir, RADIUS_CLIENTS);
 	if (read_clients_file(buffer) < 0)
 		return -1;
 	sprintf(buffer, "%.200s/%.50s", radius_dir, RADIUS_NASLIST);
-	if (read_naslist_file(buffer) < 0)
-		return -1;
+	if (stat(buffer, &st) == 0 || errno != ENOENT) {
+		if (read_naslist_file(buffer) < 0)
+			return -1;
+	}
 	sprintf(buffer, "%.200s/%.50s", radius_dir, RADIUS_REALMS);
-	if (read_realms_file(buffer) < 0)
-		return -1;
+	if (stat(buffer, &st) == 0 || errno != ENOENT) {
+		if (read_realms_file(buffer) < 0)
+			return -1;
+	}
 
 	if (cache_passwd) {
 		log(L_INFO, "HASH:  Reinitializing hash structures "
@@ -2138,4 +2241,3 @@ int read_config_files()
 
 	return 0;
 }
-
