@@ -31,7 +31,7 @@
 
 #define AUTH_VECTOR_LEN		16
 #define AUTH_PASS_LEN		16
-#define AUTH_STRING_LEN		128	/* maximum of 254 */
+#define AUTH_STRING_LEN		254	/* 253 max + trailing zero */
 
 
 typedef struct pw_auth_hdr {
@@ -48,6 +48,8 @@ typedef struct pw_auth_hdr {
 #define PW_AUTH_UDP_PORT		1645
 #define PW_ACCT_UDP_PORT		1646
 
+#define VENDORPEC_USR			429
+
 #define PW_TYPE_STRING			0
 #define PW_TYPE_INTEGER			1
 #define PW_TYPE_IPADDR			2
@@ -90,6 +92,12 @@ typedef struct pw_auth_hdr {
 #define PW_FRAMED_ROUTE			22
 #define PW_FRAMED_IPXNET		23
 #define PW_STATE			24
+#define PW_CLASS			25
+#define PW_VENDOR_SPECIFIC		26
+#define PW_SESSION_TIMEOUT		27
+#define PW_IDLE_TIMEOUT			28
+#define PW_CALLED_STATION_ID		30
+#define PW_CALLING_STATION_ID		31
 #define PW_PROXY_STATE			33
 
 #define PW_ACCT_STATUS_TYPE		40
@@ -102,11 +110,16 @@ typedef struct pw_auth_hdr {
 #define PW_ACCT_INPUT_PACKETS		47
 #define PW_ACCT_OUTPUT_PACKETS		48
 
+#define PW_CHAP_CHALLENGE		60
 #define PW_NAS_PORT_TYPE		61
 #define PW_PORT_LIMIT			62
 #define PW_CONNECT_INFO			77
 
-#define PW_HUNTGROUP_NAME		221
+#define PW_FALL_THROUGH			500
+#define PW_ADD_PORT_TO_IP_ADDRESS	501
+#define PW_EXEC_PROGRAM			502
+#define PW_EXEC_PROGRAM_WAIT		503
+
 #define PW_AUTHTYPE			1000
 #define PW_PREFIX			1003
 #define PW_SUFFIX			1004
@@ -115,13 +128,13 @@ typedef struct pw_auth_hdr {
 #define PW_CONNECT_RATE			1007
 #define PW_USER_CATEGORY		1029
 #define PW_GROUP_NAME			1030
+#define PW_HUNTGROUP_NAME		1031
 #define PW_SIMULTANEOUS_USE		1034
 #define PW_STRIP_USERNAME		1035
-#define PW_FALL_THROUGH			1036
-#define PW_ADD_PORT_TO_IP_ADDRESS	1037
-#define PW_EXEC_PROGRAM			1038
-#define PW_EXEC_PROGRAM_WAIT		1039
 #define PW_HINT				1040
+#define PAM_AUTH_ATTR			1041
+#define PW_LOGIN_TIME			1042
+#define PW_REALM			1045
 
 /*
  *	INTEGER TRANSLATIONS
@@ -165,7 +178,7 @@ typedef struct pw_auth_hdr {
 #define PW_AUTHTYPE_CRYPT		3
 #define PW_AUTHTYPE_REJECT		4
 #define PW_AUTHTYPE_PAM			253
-#define PW_AUTHTYPE_NONE		254
+#define PW_AUTHTYPE_ACCEPT		254
 
 /*	PORT TYPES		*/
 #define PW_NAS_PORT_ASYNC		0
