@@ -65,6 +65,7 @@ int			log_stripped_names;
 int 			cache_passwd = 0;
 int			debug_flag;
 int			use_dbm = 0;
+int			use_wtmp = 1;
 UINT4			myip = 0;
 UINT4			warning_seconds;
 int			log_auth_detail = 0;
@@ -161,6 +162,7 @@ int main(int argc, char **argv)
 	int			status;
 	int			dontfork = 0;
 	int			radius_port = 0;
+	int			check_config = 0;
 #ifdef RADIUS_PID
 	FILE			*fp;
 #endif
@@ -205,7 +207,7 @@ int main(int argc, char **argv)
 	/*
 	 *	Process the options.
 	 */
-	while((argval = getopt(argc, argv, "ASa:ci:l:d:bfp:svxyz")) != EOF) {
+	while((argval = getopt(argc, argv, "ACSDa:ci:l:d:bfp:svwxyz")) != EOF) {
 
 		switch(argval) {
 
@@ -226,6 +228,14 @@ int main(int argc, char **argv)
 			cache_passwd = 1;
 			break;
 
+		case 'C':
+			check_config = 1;
+			break;
+
+		case 'D':
+			use_dns = 0;
+			break;
+
 		case 'd':
 			radius_dir = optarg;
 			break;
@@ -262,6 +272,10 @@ int main(int argc, char **argv)
 			version();
 			break;
 
+		case 'w':
+			use_wtmp = 0;
+			break;
+
 		case 'x':
 			debug_flag++;
 			break;
@@ -280,6 +294,19 @@ int main(int argc, char **argv)
 		}
 	}
 
+	if (check_config) {
+		debug_flag = 1;
+#if 0
+		fclose(stderr);
+		stderr = stdout;
+#else
+		dup2(1, 2);
+#endif
+		reread_config(0);
+		printf("Configuration OK\n");
+		return 0;
+	}
+
 	/*
 	 *	Open Authentication socket.
 	 */
@@ -353,7 +380,7 @@ int main(int argc, char **argv)
 		if(pid > 0) {
 			exit(0);
 		}
-#if defined(__linux__) || defined(__svr4__)
+#if defined(__linux__) || defined(__svr4__) || defined(__hpux__)
 		setsid();
 #endif
 	}
@@ -608,13 +635,11 @@ static void rad_spawn_child(AUTH_REQ *authreq, int activefd, FUNP fun)
 			 */
 			if (prevreq == (AUTH_REQ *)NULL) {
 				first_request = curreq->next;
-				pairfree(curreq->request);
-				free(curreq);
+				authfree(curreq);
 				curreq = first_request;
 			} else {
 				prevreq->next = curreq->next;
-				pairfree(curreq->request);
-				free(curreq);
+				authfree(curreq);
 				curreq = prevreq->next;
 			}
 		} else if (curreq->ipaddr == authreq->ipaddr &&
@@ -631,8 +656,7 @@ static void rad_spawn_child(AUTH_REQ *authreq, int activefd, FUNP fun)
 				"Dropping duplicate authentication packet"
 				" from client %s - ID: %d",
 				client_name(authreq->ipaddr), authreq->id);
-				pairfree(authreq->request);
-				free(authreq);
+				authfree(authreq);
 				request_list_busy = 0;
 				sig_cleanup(SIGCHLD);
 
@@ -679,8 +703,7 @@ static void rad_spawn_child(AUTH_REQ *authreq, int activefd, FUNP fun)
 	if (request_count > MAX_REQUESTS) {
 		log(L_ERR, "Dropping request (too many): from client %s - ID: %d",
 				client_name(authreq->ipaddr), authreq->id);
-		pairfree(authreq->request);
-		free(authreq);
+		authfree(authreq);
 
 		request_list_busy = 0;
 		sig_cleanup(SIGCHLD);
